
'S2 include code version 2010.11.18

'---[Constants]----------------------------------------------------------------

CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000
  EE_REFLIGHTS  = s2#EE_USER_AREA
  LINE_THLD     = 32
  BAR_THLD      = 32
  OBSTACLE_THLD = 20
  SPKR_VOL      = 35

'---[Global Variables]---------------------------------------------------------

VAR

  long  CoinFlip, LeftMotor, RightMotor, MoveTime, pLeftMotor, pRightMotor, pMoveTime, FMStack[50]
  word  WheelSpace, FullCircle, SeqCounter
  byte  LeftLight, CenterLight, RightLight, RefLights[3]
  byte  ResetCount, LineCount, ObstacleCount, StallCount, ObstacleThld
  byte  LineThld, LeftLine, RightLine, LeftObstacle, RightObstacle, Self
  byte  Flag_green, Flag_yellow, Flag_orange, Flag_red, Flag_magenta, Flag_purple, Flag_blue, Stalled

'---[Object Declaration]-------------------------------------------------------

OBJ

  s2    : "S2"

'---[Start of Program]---------------------------------------------------------

PUB start

  s2.start
  s2.start_motors
  s2.start_tones
  s2.button_mode(true, true)
  s2.set_volume(SPKR_VOL)
  s2.set_voices(s2#SAW, s2#SAW)
  ResetCount := s2.reset_button_count
  if (s2.get_line_threshold <> s2#DEFAULT_LINE_THLD)
    LineThld := s2.get_line_threshold
  else
    LineThld := LINE_THLD 
  if (s2.get_obstacle_threshold <> s2#DEFAULT_OBSTACLE_THLD)
    ObstacleThld := s2.get_obstacle_threshold
  else
    ObstacleThld := OBSTACLE_THLD 
  waitcnt(cnt + 10_000_000) 
  CoinFlip :=  s2.light_sensor_raw(s2#LEFT) << 24 | s2.light_sensor_raw(s2#CENTER) << 12 | s2.light_sensor_raw(s2#RIGHT)
  Self := cogid
  long[@WheelSpace] := s2.get_wheel_calibration
  cognew(FaultMonitor, @FMStack)
  \Green
  repeat

'---[Battery and Over-current Monitor Cog]-------------------------------------

PUB FaultMonitor : value

  value := $ffff
  waitcnt(cnt + 80_000_000)
  repeat
    value <#= s2.get_adc_results(s2#ADC_VBAT)
    if value > constant((700*2550)/(400*33))      '7.0V
      s2.set_led(s2#POWER,s2#BLUE)
    elseif value > constant((600*2550)/(400*33))  '6.0V
      s2.set_led(s2#POWER,$20)
    else
      s2.set_led(s2#POWER,s2#BLINK_BLUE)
    if s2.get_adc_results(s2#ADC_IMOT) > 210
      cogstop(Self)
      s2.stop_now
      s2.set_leds(s2#BLINK_RED,s2#BLINK_RED,s2#BLINK_RED,s2#OFF)
      repeat

'---[Main Program: Green]------------------------------------------------------

PUB Green

  repeat
    ReadStall
    ifnot (Stalled == 1)
      MotorSet(128, 128, 0)
    else
      repeat SeqCounter from 1 to 15
        PlayPulse(1000, 0, 5)
      PlayNote(100, 0, 0)
      repeat SeqCounter from 1 to 25
        PlayPulse(1000, 0, 8)
      s2.wait_sync(0)
      MotorSet(-128, -128, 1000)
      MotorSet(128, -128, 2000)

'---[Set Motor Speeds]---------------------------------------------------------

PRI MotorSet(lmotor, rmotor, timer)

  MoveTime := timer #> 0
  LeftMotor := lmotor #> -256 <# 256
  RightMotor := rmotor #> -256 <# 256
  if (LeftMotor <> pLeftMotor or RightMotor <> pRightMotor or MoveTime <> pMoveTime or pMoveTime <> 0)
    if (MoveTime)
      s2.move_now(LeftMotor * MoveTime * FullCircle / 1_024_000, RightMotor * MoveTime * FullCircle / 1_024_000, MoveTime << 1, (||LeftMotor #> ||RightMotor <# 255) >> 4, 0)
      s2.wait_stop
    else  
      s2.wheels_now(LeftMotor, RightMotor, 0)
      waitcnt(cnt + clkfreq / 10)
    pLeftMotor := LeftMotor
    pRightMotor := RightMotor
    pMoveTime := MoveTime

'---[Play an Individual Note, Separated by a Short Delay.]---------------------

PRI PlayNote(duration, freq1, freq2)

  s2.play_tone(duration - 1 #> 1, freq1, freq2)
  s2.play_tone(1, 0, 0)

'---[Play an Individual Pulse, Followed by a Delay.]---------------------------

PRI PlayPulse(on_duration, nil, off_duration) | on_freq

  on_duration := on_duration * 2 + 1380
  on_freq := 1_000_000 / on_duration
  on_duration := on_duration / 700 #> 1
  s2.play_tone(on_duration, on_freq, 0)
  if (off_duration)
    s2.play_tone(off_duration, 0, 0)

'---[Read Stall Sensor]--------------------------------------------------------

PRI ReadStall

  if (Stalled := s2.stalled & 1)
    StallCount := (StallCount + 1) <# 8
  else
    StallCount := 0

'---[End of Program]-----------------------------------------------------------
